class Ghost{
  PImage[] left;
  PImage[] right;
  PImage[] destroyLeft;
  PImage[] destroyRight;
  PImage idleLeft;
  PImage idleRight;
  SpriteSheet sheet;
  PuzzleObject closest;
  TimeBar destroyBar;
  
  PVector movement = new PVector(0, 0);
  float moveSpeed = 4.3;
  int posX;
  int posY;
  
  boolean chaseHuman = false;
  boolean destroying = false;
  boolean moving = false;
  
  int frame = 0;
  int animSteps = 2;
  int previousTime = 1000;
  
  Ghost(int PosX, int PosY, PImage IdleLeft, PImage IdleRight, PImage Left, PImage Right, PImage DestroyLeft, PImage DestroyRight){
    posX = PosX;
    posY = PosY;
    
    idleLeft = IdleLeft;
    idleRight = IdleRight;
    
    sheet = new SpriteSheet(Left, animSteps, 1);
    left = sheet.images;
    sheet = new SpriteSheet(Right, animSteps, 1);
    right = sheet.images;
    
    sheet = new SpriteSheet(DestroyLeft, animSteps, 1);
    destroyLeft = sheet.images;
    sheet = new SpriteSheet(DestroyRight, animSteps, 1);
    destroyRight = sheet.images;
    
    destroyBar = new TimeBar(posX, posY, 100, 20, 130, red, "");
  }
  
  void update(){
    if(human.anxiety > ghostFollowAnxiety && !human.isHiding){//chaseHuman){
      destroying = false;
      chaseHuman = true;
      resetBar();
      chaseHumanAI();
    }
    else{
      chaseHuman = false;
      destroyAI();
    }
    
    if(moving || chaseHuman){
      if(movement.x < 0){
        animate(left, 500);
      }
      else{
        animate(right, 500);
      }
    }
    else if(destroying){
      if(movement.x < 0){
        animate(destroyLeft, 250);
      }
      else{
        animate(destroyRight, 250);
      }
    }
    else{
      if(movement.x < 0){
        image(idleLeft, posX, posY, 150, 150);
      }
      else{
        image(idleRight, posX, posY, 150, 150);
      }
    }
    moving = false;
  }
  
  void chaseHumanAI(){
    if(new PVector(human.posX, human.posY).dist(new PVector(posX, posY)) > 60){
      moveTo(new PVector(human.posX, human.posY));
    }
  }
  
  void destroyAI(){
    if(closest != null && closest.natualState){
      if(!destroying){
        moveTo(new PVector(closest.posX, closest.posY));
      }
      destroyObject();
      
    }
    else{
      findClosestFixedObject();
    }
  }
  
  void moveTo(PVector target){
    moving = true;
    
    movement.x = target.x - posX;
    movement.y = target.y - posY;
    
    movement.setMag(moveSpeed);
    posX += movement.x;
    posY += movement.y;
  }
  
  void destroyObject(){
    if(destroying){
      destroyBar.update();
      
      if(destroyBar.complete){
        resetBar();
        progressBar.intactObjects -= 1;
        destroying = false;
        closest.sabotage();
      }
    }
    else if(new PVector(closest.posX, closest.posY).dist(new PVector(posX, posY)) < 5){
      destroying = true;
      setBar();
    }
  }
  
  void findClosestFixedObject(){
    for(int i = 0; i < puzzleObjects.size(); i++){
      if(puzzleObjects.get(i).natualState){
        if(closest == null || !closest.natualState){
          closest = puzzleObjects.get(i);
        } //else if checks if puzzleObjects(i) is closer then the current "closest" object <3
        else if(puzzleObjects.get(i).natualState && new PVector(posX, posY).dist(new PVector(puzzleObjects.get(i).posX, puzzleObjects.get(i).posX)) < new PVector(posX, posY).dist(new PVector(closest.posX, closest.posX))){
          closest = puzzleObjects.get(i);
        }
      }
    }
  }
  
  void animate(PImage[] anim, int speed){
    if(millis() > previousTime + speed){
      if(frame < animSteps - 1){
        frame++;
      }
      else{
        frame = 0;
      }
      previousTime = millis();
    }
    //imageMode(CENTER);
    image(anim[frame], posX, posY, 150, 150); 
  }
  
  void setBar(){
    destroyBar.baseTime = millis();
    destroyBar.posX = posX + 50;
    destroyBar.posY = posY;
  }
  
  void resetBar(){
    destroyBar.progress = 0;
    destroyBar.complete = false;
  }
}
